@@ -8,8 +8,6 @@ import sys
 from logging.config import dictConfig as configure_logging_by_dict
 from warnings import warn as _warn
 
-import six
-
 from saml2 import BINDING_HTTP_ARTIFACT
 from saml2 import BINDING_HTTP_POST
 from saml2 import BINDING_HTTP_REDIRECT
@@ -76,6 +74,9 @@ COMMON_ARGS = [
     "metadata",
     "ui_info",
     "name_id_format",
+    "signing_algorithm",
+    "digest_algorithm",
+    "http_client_timeout",
 ]
 
 SP_ARGS = [
@@ -102,6 +103,7 @@ SP_ARGS = [
     "sp_type",
     "sp_type_in_metadata",
     "requested_attributes",
+    "requested_authn_context",
 ]
 
 AA_IDP_ARGS = [
@@ -225,6 +227,9 @@ class Config(object):
         self.attribute_profile = []
         self.requested_attribute_name_format = NAME_FORMAT_URI
         self.delete_tmpfiles = True
+        self.signing_algorithm = None
+        self.digest_algorithm = None
+        self.http_client_timeout = None
 
     def setattr(self, context, attr, val):
         if context == "":
@@ -370,18 +375,21 @@ class Config(object):
 
         try:
             ca_certs = self.ca_certs
-        except:
+        except Exception:
             ca_certs = None
         try:
             disable_validation = self.disable_ssl_certificate_validation
-        except:
+        except Exception:
             disable_validation = False
 
-        mds = MetadataStore(acs, self, ca_certs,
-            disable_ssl_certificate_validation=disable_validation)
-
+        mds = MetadataStore(
+            acs,
+            self,
+            ca_certs,
+            disable_ssl_certificate_validation=disable_validation,
+            http_client_timeout=self.http_client_timeout,
+        )
         mds.imp(metadata_conf)
-
         return mds
 
     def endpoint(self, service, binding=None, context=None):
@@ -400,7 +408,12 @@ class Config(object):
         if endps and service in endps:
             for endpspec in endps[service]:
                 try:
-                    endp, bind = endpspec
+                    # endspec sometime is str, sometime is a tuple
+                    if type(endpspec) in (tuple, list):
+                        # slice prevents 3-tuple, eg: sp's assertion_consumer_service
+                        endp, bind = endpspec[0:2]
+                    else:
+                        endp, bind = endpspec
                     if binding is None or bind == binding:
                         spec.append(endp)
                 except ValueError:
