@@ -30,6 +30,7 @@ from saml2.extension.idpdisc import BINDING_DISCO
 from saml2.extension.idpdisc import DiscoveryResponse
 from saml2.md import NAMESPACE as NS_MD
 from saml2.md import EntitiesDescriptor
+from saml2.md import EntityDescriptor
 from saml2.md import ArtifactResolutionService
 from saml2.md import NameIDMappingService
 from saml2.md import SingleSignOnService
@@ -44,6 +45,7 @@ from saml2.time_util import add_duration
 from saml2.time_util import before
 from saml2.time_util import str_to_time
 from saml2.validate import NotValid
+from saml2.sigver import SignatureError
 from saml2.sigver import security_context
 from saml2.extension.mdattr import NAMESPACE as NS_MDATTR
 from saml2.extension.mdattr import EntityAttributes
@@ -490,20 +492,16 @@ class MetaData(object):
         def extract_certs(srvs):
             res = []
             for srv in srvs:
-                if "key_descriptor" in srv:
-                    for key in srv["key_descriptor"]:
-                        if "use" in key and key["use"] == use:
-                            for dat in key["key_info"]["x509_data"]:
-                                cert = repack_cert(
-                                    dat["x509_certificate"]["text"])
-                                if cert not in res:
-                                    res.append(cert)
-                        elif not "use" in key:
-                            for dat in key["key_info"]["x509_data"]:
-                                cert = repack_cert(
-                                    dat["x509_certificate"]["text"])
-                                if cert not in res:
-                                    res.append(cert)
+                for key in srv.get("key_descriptor", []):
+                    key_use = key.get("use")
+                    key_info = key.get("key_info") or {}
+                    key_name = (key_info.get("key_name") or [{"text": None}])[0]
+                    key_name_txt = key_name.get("text")
+                    if "use" not in key or key_use == use:
+                        for dat in key_info["x509_data"]:
+                            cert = repack_cert(dat["x509_certificate"]["text"])
+                            if cert not in res:
+                                res.append((key_name_txt, cert))
 
             return res
 
@@ -618,7 +616,14 @@ class InMemoryMetaData(MetaData):
         try:
             self.entities_descr = md.entities_descriptor_from_string(xmlstr)
         except Exception as e:
-            raise SAMLError(f'Failed to parse metadata file: {self.filename}') from e
+            _md_desc = (
+                f'metadata file: {self.filename}'
+                if isinstance(self,MetaDataFile)
+                else f'remote metadata: {self.url}'
+                if isinstance(self, MetaDataExtern)
+                else 'metadata'
+            )
+            raise SAMLError(f'Failed to parse {_md_desc}') from e
 
         if not self.entities_descr:
             self.entity_descr = md.entity_descriptor_from_string(xmlstr)
@@ -739,14 +744,50 @@ class InMemoryMetaData(MetaData):
         if not self.signed():
             return True
 
-        fallback_name = "{ns}:{tag}".format(
-            ns=md.EntitiesDescriptor.c_namespace, tag=md.EntitiesDescriptor.c_tag
-        )
-        node_name = self.node_name or fallback_name
+        if self.node_name is not None:
+            try:
+                self.security.verify_signature(
+                    txt, node_name=self.node_name, cert_file=self.cert
+                )
+            except SignatureError as e:
+                error_context = {
+                    "message": "Failed to verify signature",
+                    "node_name": self.node_name,
+                }
+                raise SignatureError(error_context) from e
+            else:
+                return True
 
-        return self.security.verify_signature(
-            txt, node_name=node_name, cert_file=self.cert
+        def try_verify_signature(node_name):
+            try:
+                self.security.verify_signature(
+                    txt, node_name=node_name, cert_file=self.cert
+                )
+            except SignatureError as e:
+                return False
+            else:
+                return True
+
+        descriptor_names = [
+            f"{ns}:{tag}"
+            for ns, tag in [
+                (EntitiesDescriptor.c_namespace, EntitiesDescriptor.c_tag),
+                (EntityDescriptor.c_namespace, EntityDescriptor.c_tag),
+            ]
+        ]
+
+        verified_w_descriptor_name = any(
+            try_verify_signature(node_name)
+            for node_name in descriptor_names
         )
+        if not verified_w_descriptor_name:
+            error_context = {
+                "message": "Failed to verify signature",
+                "descriptor_names": descriptor_names,
+            }
+            raise SignatureError(error_context)
+
+        return verified_w_descriptor_name
 
 
 class MetaDataFile(InMemoryMetaData):
@@ -887,7 +928,8 @@ class MetaDataMDX(InMemoryMetaData):
         return transform
 
     def __init__(self, url=None, security=None, cert=None,
-                 entity_transform=None, freshness_period=None, **kwargs):
+                 entity_transform=None, freshness_period=None,
+                 http_client_timeout=None, **kwargs):
         """
         :params url: mdx service url
         :params security: SecurityContext()
@@ -899,6 +941,7 @@ class MetaDataMDX(InMemoryMetaData):
         sha1 transformation.
         :params freshness_period: a duration in the format described at
         https://www.w3.org/TR/xmlschema-2/#duration
+        :params http_client_timeout: timeout of http requests
         """
         super(MetaDataMDX, self).__init__(None, **kwargs)
         if not url:
@@ -915,6 +958,7 @@ class MetaDataMDX(InMemoryMetaData):
         self.security = security
         self.freshness_period = freshness_period or DEFAULT_FRESHNESS_PERIOD
         self.expiration_date = {}
+        self.http_client_timeout = http_client_timeout
 
         # We assume that the MDQ server will return a single entity
         # described by a single <EntityDescriptor> element. The protocol
@@ -923,7 +967,7 @@ class MetaDataMDX(InMemoryMetaData):
         # that use case since it is unlikely to be leveraged for most
         # flows.
         self.node_name = "{ns}:{tag}".format(
-            ns=md.EntityDescriptor.c_namespace, tag=md.EntityDescriptor.c_tag
+            ns=EntityDescriptor.c_namespace, tag=EntityDescriptor.c_tag
         )
 
     def load(self, *args, **kwargs):
@@ -935,7 +979,8 @@ class MetaDataMDX(InMemoryMetaData):
             url=self.url, id=self.entity_transform(item)
         )
 
-        response = requests.get(mdx_url, headers={"Accept": SAML_METADATA_CONTENT_TYPE})
+        response = requests.get(mdx_url, headers={"Accept": SAML_METADATA_CONTENT_TYPE},
+                                timeout=self.http_client_timeout)
         if response.status_code != 200:
             error_msg = "Fething {item}: Got response status {status}".format(
                 item=item, status=response.status_code
@@ -981,7 +1026,7 @@ class MetadataStore(MetaData):
     def __init__(self, attrc, config, ca_certs=None,
                  check_validity=True,
                  disable_ssl_certificate_validation=False,
-                 filter=None):
+                 filter=None, http_client_timeout=None):
         """
         :params attrc:
         :params config: Config()
@@ -991,9 +1036,9 @@ class MetadataStore(MetaData):
         MetaData.__init__(self, attrc, check_validity=check_validity)
 
         if disable_ssl_certificate_validation:
-            self.http = HTTPBase(verify=False, ca_bundle=ca_certs)
+            self.http = HTTPBase(verify=False, ca_bundle=ca_certs, http_client_timeout=http_client_timeout)
         else:
-            self.http = HTTPBase(verify=True, ca_bundle=ca_certs)
+            self.http = HTTPBase(verify=True, ca_bundle=ca_certs, http_client_timeout=http_client_timeout)
 
         self.security = security_context(config)
         self.ii = 0
@@ -1001,6 +1046,7 @@ class MetadataStore(MetaData):
         self.check_validity = check_validity
         self.filter = filter
         self.to_old = {}
+        self.http_client_timeout = http_client_timeout
 
     def load(self, *args, **kwargs):
         if self.filter:
@@ -1059,16 +1105,29 @@ class MetadataStore(MetaData):
                 security = self.security
                 entity_transform = kwargs.get('entity_transform', None)
                 _md = MetaDataMDX(url, security, cert, entity_transform,
-                                  freshness_period=freshness_period)
+                                  freshness_period=freshness_period,
+                                  http_client_timeout=self.http_client_timeout)
             else:
                 key = args[1]
                 url = args[1]
-                _md = MetaDataMDX(url)
+                _md = MetaDataMDX(url, http_client_timeout=self.http_client_timeout)
         else:
             raise SAMLError("Unknown metadata type '%s'" % typ)
         _md.load()
         self.metadata[key] = _md
 
+    def reload(self, spec):
+        # Save the old set of metadata
+        old_metadata = self.metadata
+        self.metadata = {}
+        try:
+            # Reload the metadata based on the spec
+            self.imp(spec)
+        except Exception as e:
+            # Something went wrong, restore the previous metadata
+            self.metadata = old_metadata
+            raise e
+
     def imp(self, spec):
         # This serves as a backwards compatibility
         if type(spec) is dict:
@@ -1131,8 +1190,7 @@ class MetadataStore(MetaData):
 
     def service(self, entity_id, typ, service, binding=None):
         known_entity = False
-        logger.debug("service(%s, %s, %s, %s)", entity_id, typ, service,
-                     binding)
+        logger.debug("service(%s, %s, %s, %s)", entity_id, typ, service, binding)
         for key, _md in self.metadata.items():
             srvs = _md.service(entity_id, typ, service, binding)
             if srvs:
@@ -1233,8 +1291,6 @@ class MetadataStore(MetaData):
         # IDP + SP
         if typ is None:
             raise AttributeError("Missing type specification")
-        if binding is None:
-            binding = BINDING_HTTP_REDIRECT
         return self.service(entity_id, "%s_descriptor" % typ,
                             "single_logout_service", binding)
 
@@ -1378,13 +1434,15 @@ class MetadataStore(MetaData):
             ext = self.__getitem__(entity_id)["extensions"]
         except KeyError:
             return res
+
         for elem in ext["extension_elements"]:
-            if elem["__class__"] == classnames["mdattr_entityattributes"]:
-                for attr in elem["attribute"]:
-                    if attr["name"] not in res:
-                        res[attr["name"]] = []
-                    res[attr["name"]] += [v["text"] for v in attr[
-                        "attribute_value"]]
+            if elem["__class__"] != classnames["mdattr_entityattributes"]:
+                continue
+            for attr in elem["attribute"]:
+                res[attr["name"]] = [
+                    *res.get(attr["name"], []),
+                    *(v["text"] for v in attr.get("attribute_value", []))
+                ]
         return res
 
     def supported_algorithms(self, entity_id):
@@ -1463,6 +1521,29 @@ class MetadataStore(MetaData):
         }
         return res
 
+    def registration_info_typ(self, entity_id, typ):
+        try:
+            md = self.__getitem__(entity_id)
+        except KeyError:
+            md = {}
+
+        services_of_type = md.get(typ) or []
+        typ_reg_info = (
+            {
+                "registration_authority": elem.get("registration_authority"),
+                "registration_instant": elem.get("registration_instant"),
+                "registration_policy": {
+                    policy["lang"]: policy["text"]
+                    for policy in elem.get("registration_policy", [])
+                    if policy.get("__class__") == classnames["mdrpi_registration_policy"]
+                },
+            }
+            for srv in services_of_type
+            for elem in srv.get("extensions", {}).get("extension_elements", [])
+            if elem.get("__class__") == classnames["mdrpi_registration_info"]
+        )
+        return typ_reg_info
+
     def _lookup_elements_by_cls(self, root, cls):
         elements = (
             element
@@ -1484,6 +1565,15 @@ class MetadataStore(MetaData):
         return elements
 
     def sbibmd_scopes(self, entity_id, typ=None):
+        warn_msg = (
+            "`saml2.mdstore.MetadataStore::sbibmd_scopes` method is deprecated; "
+            "instead, use `saml2.mdstore.MetadataStore::shibmd_scopes`."
+        )
+        logger.warning(warn_msg)
+        _warn(warn_msg, DeprecationWarning)
+        return self.shibmd_scopes(entity_id, typ=typ)
+
+    def shibmd_scopes(self, entity_id, typ=None):
         try:
             md = self[entity_id]
         except KeyError:
@@ -1695,4 +1785,5 @@ class MetadataStore(MetaData):
 
             return "%s" % res
         elif format == "md":
-            return json.dumps(self.items(), indent=2)
+            # self.items() returns dictitems(), convert that back into a dict
+            return json.dumps(dict(self.items()), indent=2)
