@@ -67,6 +67,10 @@ class IncorrectlySigned(SAMLError):
     pass
 
 
+class InvalidAssertion(SAMLError):
+    pass
+
+
 class DecryptionFailed(SAMLError):
     pass
 
@@ -94,6 +98,9 @@ class StatusAuthnFailed(StatusError):
 class StatusInvalidAttrNameOrValue(StatusError):
     pass
 
+class StatusInvalidAuthnResponseStatement(StatusError):
+    pass
+
 
 class StatusInvalidNameidPolicy(StatusError):
     pass
@@ -209,10 +216,10 @@ def for_me(conditions, myself):
         if not restriction.audience:
             continue
         for audience in restriction.audience:
-            if audience.text.strip() == myself:
+            if audience.text and audience.text.strip() == myself:
                 return True
             else:
-                logger.debug("AudienceRestriction - One condition not satisfied: %s != %s" % (audience.text.strip(), myself))
+                logger.debug("AudienceRestriction - One condition not satisfied: {} != {}".format(audience.text, myself))
     logger.debug("AudienceRestrictions not satisfied!")
     return False
 
@@ -255,7 +262,7 @@ class StatusResponse(object):
     def __init__(self, sec_context, return_addrs=None, timeslack=0,
             request_id=0, asynchop=True, conv_info=None):
         self.sec = sec_context
-        self.return_addrs = return_addrs
+        self.return_addrs = return_addrs or []
 
         self.timeslack = timeslack
         self.request_id = request_id
@@ -394,9 +401,7 @@ class StatusResponse(object):
                          self.in_response_to, self.request_id)
             return None
 
-        try:
-            assert self.response.version == "2.0"
-        except AssertionError:
+        if self.response.version != "2.0":
             _ver = float(self.response.version)
             if _ver < 2.0:
                 raise RequestVersionTooLow()
@@ -404,15 +409,15 @@ class StatusResponse(object):
                 raise RequestVersionTooHigh()
 
         if self.asynchop:
-            if self.response.destination and \
-                            self.response.destination not in self.return_addrs:
-                logger.error("%s not in %s", self.response.destination,
-                             self.return_addrs)
+            if (
+                self.response.destination
+                and self.response.destination not in self.return_addrs
+            ):
+                logger.error("destination '%s' not in return addresses '%s'", self.response.destination, self.return_addrs)
                 return None
 
-        assert self.issue_instant_ok()
-        assert self.status_ok()
-        return self
+        valid = self.issue_instant_ok() and self.status_ok()
+        return valid
 
     def loads(self, xmldata, decode=True, origxml=None):
         return self._loads(xmldata, decode, origxml)
@@ -430,7 +435,12 @@ class StatusResponse(object):
         self.response = mold.response
 
     def issuer(self):
-        return self.response.issuer.text.strip()
+        issuer_value = (
+            self.response.issuer.text
+            if self.response.issuer is not None
+            else ""
+        ).strip()
+        return issuer_value
 
 
 class LogoutResponse(StatusResponse):
@@ -509,9 +519,7 @@ class AuthnResponse(StatusResponse):
     def check_subject_confirmation_in_response_to(self, irp):
         for assertion in self.response.assertion:
             for _sc in assertion.subject.subject_confirmation:
-                try:
-                    assert _sc.subject_confirmation_data.in_response_to == irp
-                except AssertionError:
+                if _sc.subject_confirmation_data.in_response_to != irp:
                     return False
 
         return True
@@ -526,8 +534,6 @@ class AuthnResponse(StatusResponse):
                 try:
                     if not self.check_subject_confirmation_in_response_to(
                             self.in_response_to):
-                        logger.exception(
-                            "Unsolicited response %s" % self.in_response_to)
                         raise UnsolicitedResponse(
                             "Unsolicited response: %s" % self.in_response_to)
                 except AttributeError:
@@ -536,8 +542,6 @@ class AuthnResponse(StatusResponse):
                 # Should check that I haven't seen this before
                 pass
             else:
-                logger.exception(
-                    "Unsolicited response %s" % self.in_response_to)
                 raise UnsolicitedResponse(
                     "Unsolicited response: %s" % self.in_response_to)
 
@@ -550,15 +554,13 @@ class AuthnResponse(StatusResponse):
         self.assertion = None
 
     def authn_statement_ok(self, optional=False):
-        try:
-            # the assertion MUST contain one AuthNStatement
-            assert len(self.assertion.authn_statement) == 1
-        except AssertionError:
+        n_authn_statements = len(self.assertion.authn_statement)
+        if n_authn_statements != 1:
             if optional:
                 return True
             else:
-                logger.error("No AuthnStatement")
-                raise
+                msg = "Invalid number of AuthnStatement found in Response: {n}".format(n=n_authn_statements)
+                raise ValueError(msg)
 
         authn_statement = self.assertion.authn_statement[0]
         if authn_statement.session_not_on_or_after:
@@ -626,7 +628,7 @@ class AuthnResponse(StatusResponse):
 
         return True
 
-    def decrypt_attributes(self, attribute_statement):
+    def decrypt_attributes(self, attribute_statement, keys=None):
         """
         Decrypts possible encrypted attributes and adds the decrypts to the
         list of attributes.
@@ -640,11 +642,11 @@ class AuthnResponse(StatusResponse):
 
         for encattr in attribute_statement.encrypted_attribute:
             if not encattr.encrypted_key:
-                _decr = self.sec.decrypt(encattr.encrypted_data)
+                _decr = self.sec.decrypt_keys(encattr.encrypted_data, keys=keys)
                 _attr = attribute_from_string(_decr)
                 attribute_statement.attribute.append(_attr)
             else:
-                _decr = self.sec.decrypt(encattr)
+                _decr = self.sec.decrypt_keys(encattr, keys=keys)
                 enc_attr = encrypted_attribute_from_string(_decr)
                 attrlist = enc_attr.extensions_as_elements("Attribute", saml)
                 attribute_statement.attribute.extend(attrlist)
@@ -668,9 +670,11 @@ class AuthnResponse(StatusResponse):
                 if _assertion.advice.assertion:
                     for tmp_assertion in _assertion.advice.assertion:
                         if tmp_assertion.attribute_statement:
-                            assert len(tmp_assertion.attribute_statement) == 1
-                            ava.update(self.read_attribute_statement(
-                                tmp_assertion.attribute_statement[0]))
+                            n_attr_statements = len(tmp_assertion.attribute_statement)
+                            if n_attr_statements != 1:
+                                msg = "Invalid number of AuthnStatement found in Response: {n}".format(n=n_attr_statements)
+                                raise ValueError(msg)
+                            ava.update(self.read_attribute_statement(tmp_assertion.attribute_statement[0]))
             if _assertion.attribute_statement:
                 logger.debug("Assertion contains %s attribute statement(s)",
                              (len(self.assertion.attribute_statement)))
@@ -730,10 +734,19 @@ class AuthnResponse(StatusResponse):
 
         return has_keyinfo
 
-    def get_subject(self):
+    def get_subject(self, keys=None):
         """ The assertion must contain a Subject
         """
-        assert self.assertion.subject
+
+        if not self.assertion:
+            raise ValueError("Missing assertion")
+
+        if not self.assertion.subject:
+            raise ValueError(
+                "Invalid assertion subject: {subject}".format(
+                    subject=self.assertion.subject
+                )
+            )
         subject = self.assertion.subject
         subjconf = []
 
@@ -772,8 +785,9 @@ class AuthnResponse(StatusResponse):
             self.name_id = subject.name_id
         elif subject.encrypted_id:
             # decrypt encrypted ID
-            _name_id_str = self.sec.decrypt(
-                subject.encrypted_id.encrypted_data.to_string())
+            _name_id_str = self.sec.decrypt_keys(
+                subject.encrypted_id.encrypted_data.to_string(), keys=keys
+            )
             _name_id = saml.name_id_from_string(_name_id_str)
             self.name_id = _name_id
 
@@ -819,7 +833,7 @@ class AuthnResponse(StatusResponse):
 
         # if self.context == "AuthnReq" or self.context == "AttrQuery":
         #    self.ava = self.get_identity()
-        #    logger.debug("--- AVA: %s", self.ava)
+        #    logger.debug("--- AVA: {0}".format(self.ava))
 
         try:
             self.get_subject()
@@ -920,14 +934,14 @@ class AuthnResponse(StatusResponse):
             pass
         else:
             # This is a saml2int limitation
-            try:
-                assert (
-                    len(self.response.assertion) == 1
-                    or len(self.response.encrypted_assertion) == 1
-                    or self.assertion is not None
+            n_assertions = len(self.response.assertion)
+            n_assertions_enc = len(self.response.encrypted_assertion)
+            if n_assertions != 1 and n_assertions_enc != 1 and self.assertion is None:
+                raise InvalidAssertion(
+                    "Invalid number of assertions in Response: {n}".format(
+                        n=n_assertions+n_assertions_enc
+                    )
                 )
-            except AssertionError:
-                raise Exception("No assertion part")
 
         if self.response.assertion:
             logger.debug("***Unencrypted assertion***")
@@ -945,7 +959,7 @@ class AuthnResponse(StatusResponse):
             while self.find_encrypt_data(resp) and decr_text_old != decr_text:
                 decr_text_old = decr_text
                 try:
-                    decr_text = self.sec.decrypt_keys(decr_text, keys)
+                    decr_text = self.sec.decrypt_keys(decr_text, keys=keys)
                 except DecryptError as e:
                     continue
                 else:
@@ -968,7 +982,7 @@ class AuthnResponse(StatusResponse):
             ) and decr_text_old != decr_text:
                 decr_text_old = decr_text
                 try:
-                    decr_text = self.sec.decrypt_keys(decr_text, keys)
+                    decr_text = self.sec.decrypt_keys(decr_text, keys=keys)
                 except DecryptError as e:
                     continue
                 else:
@@ -1022,7 +1036,7 @@ class AuthnResponse(StatusResponse):
 
         if self.context == "AuthnReq" or self.context == "AttrQuery":
             self.ava = self.get_identity()
-            logger.debug("--- AVA: %s", self.ava)
+            logger.debug("--- AVA: {0}".format(self.ava))
 
         return True
 
@@ -1040,7 +1054,7 @@ class AuthnResponse(StatusResponse):
             logger.error("Verification error on the response: %s", err)
             raise
         else:
-            if res is None:
+            if not res:
                 return None
 
         if not isinstance(self.response, samlp.Response):
@@ -1062,23 +1076,25 @@ class AuthnResponse(StatusResponse):
 
     def authn_info(self):
         res = []
-        for astat in self.assertion.authn_statement:
-            context = astat.authn_context
-            try:
-                authn_instant = astat.authn_instant
-            except AttributeError:
-                authn_instant = ""
-            if context:
-                try:
-                    aclass = context.authn_context_class_ref.text
-                except AttributeError:
-                    aclass = ""
-                try:
-                    authn_auth = [a.text for a in
-                                  context.authenticating_authority]
-                except AttributeError:
-                    authn_auth = []
-                res.append((aclass, authn_auth, authn_instant))
+        for statement in getattr(self.assertion, 'authn_statement', []):
+            authn_instant = getattr(statement, "authn_instant", "")
+
+            context = statement.authn_context
+            if not context:
+                continue
+
+            authn_class = (
+                getattr(context.authn_context_class_ref, "text", None)
+                or getattr(context.authn_context_decl_ref, "text", None)
+                or ""
+            )
+
+            authenticating_authorities = getattr(
+                context, "authenticating_authority", []
+            )
+            authn_auth = [authority.text for authority in authenticating_authorities]
+
+            res.append((authn_class, authn_auth, authn_instant))
         return res
 
     def authz_decision_info(self):
@@ -1102,12 +1118,16 @@ class AuthnResponse(StatusResponse):
             return {"name_id": self.name_id, "came_from": self.came_from,
                     "issuer": self.issuer(), "not_on_or_after": nooa,
                     "authz_decision_info": self.authz_decision_info()}
-        else:
+        elif getattr(self.assertion, 'authn_statement', None):
             authn_statement = self.assertion.authn_statement[0]
             return {"ava": self.ava, "name_id": self.name_id,
                     "came_from": self.came_from, "issuer": self.issuer(),
                     "not_on_or_after": nooa, "authn_info": self.authn_info(),
                     "session_index": authn_statement.session_index}
+        else:
+            raise StatusInvalidAuthnResponseStatement(
+                "The Authn Response Statement is not valid"
+            )
 
     def __str__(self):
         return self.xmlstr
